% Analog to Digital and DSP signal processing chain simulation
%
% Constants and setup
% Output register dimensional setup
Pix_output = zeros(1,Pixels);
% Compute AD conversion resolution
% resolution of coarse A-D
AD_Coarse_res = D_Vref / (2^D_Coarse_AD_bits);
% resolution of fine A-D
AD_Fine_res = D_Vref / (2^D_Fine_AD_bits);
% Slow Coarse resolution DA converter to remove RESET bias
DA_Coarse_res = D_Vref / (2^D_Coarse_DA_bits);
% Fast fine resolution DA converter to remove SIGNAL bias
DA_Fine_res = D_Vref / (2^D_Fine_DA_bits);
% calculate conversion factor for display in electrons
conv = V_e / (AD_Fine_res / (D_preamp_gain * D_Fine_gain));

% ------ Processing starts here ------
% Preamp gain and reset offset subtraction
%
D_gross_offset = (Vrst - (D_gross_bias / D_Coarse_AD_gain / D_preamp_gain)) / (DA_Coarse_res * D_Coarse_DA_Gain);
if D_gross_offset >= (2^D_Coarse_DA_bits)
  fprintf(lp,'\n !!! Coarse D/A Overflow in RESET Subtraction (Data value = %u)', D_gross_offset);
end 
sig1 = (sig - (D_gross_offset * DA_Coarse_res * D_Coarse_DA_Gain)) * D_preamp_gain;

% add johnson noise of preamplifier
% To come later !!!

% Form vectors of signal portions to sample
%

Rst_str = fix((T_Clk + T_Rstl) / m * Ns);
Rst_end = fix(Rst_str + (T_Rst / m * Ns));
Sig_str = fix(Sig_start + ((T_Xfer + T_Sstl) / m * Ns));
Sig_end = fix(Sig_str + (T_Sig / m * Ns));

Rst_Vec = sig1(Rst_str:Rst_end,1:Pixels);
Sig_Vec = sig1(Sig_str:Sig_end,1:Pixels);

fprintf(lp,'\nConversion factor for this profile is %5.3f e- / ADU', conv);
if diag_on > 0
  fprintf(lp,'\nCoarse D/A adjustment set to %5.3f volts', ((D_gross_offset * DA_Coarse_res * D_Coarse_DA_Gain)/ m));
  plot(sig1(:,2));
  j = mean(sig1(:,2));
  line([Rst_str,Rst_end],[j,j],'color','r');
  line([Sig_str,Sig_end],[j,j],'color','g');
  text(Rst_str,j,' RESET Sample');
  text(Sig_str,j,' VIDEO Sample');
  title('Timing Relationship of sampling on signal');
  fprintf(lp,'\nPaused for diagnostic - Sample Timing ... press enter to continue\n');
  if diag_on > 1
    print
  else
    pause
  end
end

% sample the signal thru hybrid a/d 
%
% how do I do this in vector math ????
% Process each pixel in the Reset and Signal arrays
  for i = 1 : Pixels
    % Process Reset portion of waveform
    % add johnson noise of coarse gain amp
    % To come later !!!
    % Trigger coarse A/D reading. 
    Coarse_AD_Bits = round(mean(Rst_Vec(1:round(D_Aperture_coarse / m * Ns),i) * D_Coarse_AD_gain / AD_Coarse_res));
    if (Coarse_AD_Bits >= (2^D_Coarse_AD_bits))
      fprintf(lp,'\n!!! Coarse A/D Overflow - Value is %u in Rst sample of pixel %u !!!', Coarse_AD_Bits,i);
    end
    % Coarse_AD_Bits represents output value of coarse conversion. 
    % Compute Coarse_DA value for offsetting to to remove bulk signal for fine A/D
    % need to protect against negative slides !
    if(Coarse_AD_Bits >= (2^D_slide))
      D_DA_Offset = Coarse_AD_Bits - (2^D_slide);
    else 
      D_DA_Offset = 0;
    end
    if (D_DA_Offset > (2^D_Fine_DA_bits))
      fprintf(lp,'\n !!! Fine D/A Overflow in Rst Processing (Data value = %u in pixel %u)', D_DA_Offset,i);
    end 

    D_DA_Offset_Voltage = 2 * D_DA_Offset * DA_Fine_res;

    % Apply the offset correction to the samples and amplify by fine gain stage
    o = (Rst_Vec(:,i) - D_DA_Offset_Voltage) * D_Fine_gain;
 
    % add johnson noise of fine gain amp
    % To come later !!!

   % Resample the data at the correct fine stage sample rate
    Sample_start = fix((D_Aperture_coarse + D_Coarse_AD_time + D_Fine_DA_settle) / m * Ns);
    pro = round(resample(o(Sample_start:size(o)),D_sample_freq,Ns) ./ AD_Fine_res);
    if (max(pro) >= (2^D_Fine_AD_bits))
      fprintf(lp,'\n!!! Fine A/D Overflow in Rst samples (data value = %u in Pixel %u) !!!',max(pro),i);
    end
    
    Rst_rslt(:,i) = pro + (Coarse_AD_Bits * D_Gain_Factor);

    % Now process the signal portion of the waveform
    % Trigger coarse A/D reading. 
    % add johnson noise of coarse gain amp
    % To come later !!!
    Coarse_AD_Bits = round(mean(Sig_Vec(1:round(D_Aperture_coarse / m * Ns),i) * D_Coarse_AD_gain / AD_Coarse_res));
    if (Coarse_AD_Bits >= (2^D_Coarse_AD_bits))
      fprintf(lp,'\n!!! Coarse A/D Overflow - Value is %u in Sig sample of pixel %u !!!', Coarse_AD_Bits,i);
    end

    % Coarse_AD_Bits represents output value of coarse conversion. 
    % Compute Coarse_DA value for offsetting to to remove bulk signal for fine A/D
    % need to protect against negative slides !
    if(Coarse_AD_Bits >= (2^D_slide))
      D_DA_Offset = Coarse_AD_Bits - (2^D_slide);
    else 
      D_DA_Offset = 0;
    end
    if (D_DA_Offset > (2^D_Fine_DA_bits))
      fprintf(lp,'\n !!! Fine D/A Overflow in Sig Processing (Data value = %u in pixel %u)', D_DA_Offset,i);
    end 

    D_DA_Offset_Voltage = 2 * D_DA_Offset * DA_Fine_res;

    % Apply the offset correction to the samples and amplify by fine gain stage
    o = (Sig_Vec(:,i) - D_DA_Offset_Voltage) * D_Fine_gain;

    % add johnson noise of fine gain amp
    % To come later !!!

    % Resample the data at the correct fine stage sample rate
    Sample_start = fix((D_Aperture_coarse + D_Coarse_AD_time + D_Fine_DA_settle) / m * Ns);
    pro = round(resample(o(Sample_start:size(o)),D_sample_freq,Ns) ./ AD_Fine_res);
    if (max(pro) >= (2^D_Fine_AD_bits))
      fprintf(lp,'\n!!! Fine A/D Overflow Sig samples (data value = %u in Pixel %u) !!!',max(pro),i);
    end

    Sig_rslt(:,i) = pro + (Coarse_AD_Bits * D_Gain_Factor);

    % simplistic decimation filter, reset subtraction and conversion to electrons
    Pix_output(1,i) = (mean(Rst_rslt(:,i)) - mean(Sig_rslt(:,i))) / conv;
  end

