% signal routines for setup of data sets

% generate data matrix
% produces a raw clean ccd signal with zero sig in
% first and last pixels. Alternate column vectors refer
% to reset and signal portions of pixels.
% Rows are sample values of the pixels in time
% reset value is Vrst and signal is Vrst - Vsig.
% first and last pixel are zero sig for reference. 

%create the real signal matrix
%
sig = zeros(Sig_len, Pixels);

% create a temporary matrix for sig values
%
s = zeros(Sig_len,1) + Vrst;

% setup first and last pixels as reference (i.e. zero charge)
sig(:,1) = s(:,1);
sig(:,Pixels) = s(:,1);

% generate signal
%
s(Sig_start:Sig_len,1) = s(Sig_start:Sig_len,1) - Vsig;

%fill the matrix .... quickest way to do this ?
for i = 2: Pixels - 1;
  sig(:,i) = s(:,1);
end
clear s;

if ADD_N_ClockFeedthru > 0
  % add clock feed through signal
  %
  % form the impulse shape
  q = 125;
  y = zeros(1000,1);
  y(1:q,1) = y(1:q,1) + 1.0;
  y(q:q * 2 - 1,1) = y(q:q * 2 - 1,1) + 2.0;
  y(q * 2:q * 3 - 1,1) = y(q * 2:q * 3 - 1,1) + 2.5;
  y(q * 3:q * 4 - 1,1) = y(q * 3:q * 4 - 1,1) - 1.0;
  y(q * 4:q * 5 - 1,1) = y(q * 4:q * 5 - 1,1) + 2.5;
  y(q * 5:q * 6 - 1,1) = y(q * 5:q * 6 - 1,1) - 1.5;
  y(q * 6:q * 7 - 1,1) = y(q * 6:q * 7 - 1,1) + 0.5;
  % filter it to make it ring
  %
  [b,a] = butter(6, Nf / (Nf * 20));
  z = filtfilt(b,a,y(:,1));
  % fit it into the clock feedthrough time slot
  Clk_sample = round((T_Clk / m) / (1 / Ns));
  r = resample(z,Clk_sample,1000);
  % set its amplitude and mix with sig matrix
  r = r .* Volts_feedthru;
  for i = 1: Pixels
    sig(1:Clk_sample,i) = sig(1:Clk_sample,i) + r(1:Clk_sample,1);
  end
  clear r;

  % now the charge dump transient
  %
  % form the impulse shape
  q = 125;
  y = zeros(1000,1);
  y(1:q,1) = y(1:q,1) - 0.3;
  y(q:q * 2 - 1,1) = y(q:q * 2 - 1,1) - 0.7;
  y(q * 2:q * 3 - 1,1) = y(q * 2:q * 3 - 1,1) - 1.0;
  y(q * 3:q * 4 - 1,1) = y(q * 3:q * 4 - 1,1) - 0.5;
  y(q * 4:q * 5 - 1,1) = y(q * 4:q * 5 - 1,1) + 0.0;
  y(q * 5:q * 6 - 1,1) = y(q * 5:q * 6 - 1,1) + 0.5;
  y(q * 6:q * 7 - 1,1) = y(q * 6:q * 7 - 1,1) + 0.3;
  % filter it to make it ring
  %
  [b,a] = butter(6, Nf / (Nf * 3));
  z = filtfilt(b,a,y(:,1));
  % fit it into the clock feedthrough time slot
  Clk_sample = round((T_Xfer / m) / (1 / Ns));
  r = resample(z,Clk_sample,1000);
  % set its amplitude and mix with sig matrix
  r = r .* Volts_feedthru;
  for i = 1: Pixels
    sig(Sig_start:Sig_start + Clk_sample - 1,i) = sig(Sig_start:Sig_start + Clk_sample - 1,i) + r(1:Clk_sample,1);
  end
  % clean up some space
  %
  clear z;
  clear y;
  clear r;
end 
% End of clock feedthrough sources

if ADD_N_Johnson > 0
  % generate Johnson noise matrix
  % same size and format as signal matrix
  % and at signal level normalized to 1
  %
  NJohn = randn(size(sig));
else
  NJohn = zeros(size(sig));
end

Noof = zeros(size(sig));
if ADD_N_OneOverF > 0
  % generate 1/f noise matrix by passing the white noise
  % matrix through a low order low pass filter
  %
  p = randn(size(sig));
  [q,r] = butter(2,oof_rolloff / Nf);
  for i = 1:Pixels
    Noof(:,i) = filter(q,r,p(:,i)) * 10;
  end
  clear p;
  clear q;
  clear r;
end

if ADD_N_Reset > 0
   % generate the reset source ripple noise
  t = 0: size(sig(:)) - 1;
  Nrst = (sin(2 * pi * Hzrstr * t));
  Nrst = reshape(Nrst, size(sig));
  Nrst = (Nrst .* Vrst * 10);
  clear t;
else
Nrst = zeros(size(sig));
end

if diag_on > 0
  q = sig(:,2);
  t = 0: (1 / (Ns / 1000000)): 1/(Pr / 1000000);
  plot(t(1:length(q)),q,'g');
  title('Waveform data for typical pixel');
  xlabel('micro secs.');
  ylabel('micro volts');
  if diag_on < 2 
    fprintf(lp,'\nPaused for diagnostic - Waveform display ... press enter to continue\n');
    pause
  else    
    print;
  end
  z1 = abs(fft(q)) * 2 / length(q);
  q = NJohn(:,1:2) .* VonJ;
  z2 = abs(fft(q)) * 2 / length(q);
  q = Noof(:,1) .* Vonoof; 
  z3 = abs(fft(q)) * 2 / length(q);
  q = Nrst(:,1) .* Vonoof; 
  z4 = abs(fft(q)) * 2 / length(q);
  f = Nf * 2 * (0:length(q) / 2 - 1) / length(q);
  semilogy(f,z1(1:length(q) / 2),'w',f,z2(1:length(q) / 2),'r',f,z3(1:length(q) / 2),'g',f,z4(1:length(q) / 2),'m');
  title('Relative spectral power of sig & noise sources at detector');
  xlabel('Frequency');
  ylabel('Relative power');
  q = [];
  f = [];
  z1 = [];
  clear z2;
  clear z3;
  clear z4;
  clear t;
  if diag_on < 2 
    fprintf(lp,'\nPaused for diagnostic - relative specral powers... press enter to continue\n');
    pause;
  else  
    print;
  end
end

% sum the noise sources to the signal
%

if ADD_N_Johnson > 0
  sig = sig + (NJohn .* VonJ);
end
if ADD_N_OneOverF > 0
  sig = sig + (Noof .* Vonoof);
end
if ADD_N_Reset > 0
  sig = sig + Nrst;
  clear Nrst;
end

if diag_on > 0
  q = sig(:,2);
  t = 0: (1 / (Ns / m)): 1/(Pr / m);
  plot(t(1:length(q)),q,'g');
  title('Waveform data for typical pixel');
  xlabel('micro secs.');
  ylabel('micro volts');
  if diag_on < 2 
    fprintf(lp,'\nPaused for diagnostic - Waveform display ... press enter to continue\n');
    pause
  else    
    print;
  end
  q = sig(:,1);
  z1 = abs(fft(q)) * 2 / length(q);
  f = Nf * 2 * (0:length(q) / 2 - 1) / length(q);
  semilogy(f,z1(1:length(q) / 2));
  title('Spectral content of generated sig + noise sources at detector');
  xlabel('Frequency');
  ylabel('Power');
  clear q;
  clear f;
  clear z1;
  if diag_on > 1 
    print; 
  end
end

% Save the data files for reference
  a = [Data_directory, '\Nsig_', int2str(Pro_file), '.m'];
  q = fopen(a,'w');
  fwrite(q,sig,'double');
  fclose(q);
  fprintf(lp,'\nSignal data saved as %s',a);
  a = [Data_directory, '\NJohn_', int2str(Pro_file), '.m'];
  q = fopen(a,'w');
  fwrite(q,NJohn,'double');
  fclose(q);
  fprintf(lp,'\nJohnson noise data saved as %s',a);
  a = [Data_directory, '\Noof_', int2str(Pro_file), '.m'];
  q = fopen(a,'w');
  fwrite(q,Noof,'double');
  fclose(q);
  fprintf(lp,'\n1/f noise data saved as %s\n\n',a);
  clear a;
